# Copyright (C) 2019  Nexedi SA and Contributors.
#                     Romain Courteaud <romain@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

import unittest
from surykatka.db import LogDB
import peewee
import surykatka.dns
from surykatka.dns import (
    expandDomainList,
    packDns,
    logDnsQuery,
    buildResolver,
    queryDNS,
    getReachableResolverList,
    getDomainIpDict,
)
from surykatka.status import logStatus
import mock
from dns import name as dns_name


class MockAnswerA(object):
    def __init__(self, address):
        self.address = address


class MockAnswerMX(object):
    def __init__(self, label):
        self.exchange = dns_name.Name(dns_name.from_text(label))


class MockAnswerTXT(object):
    def __init__(self, text):
        self.text = text

    def to_text(self):
        return self.text


class SurykatkaDNSTestCase(unittest.TestCase):
    def setUp(self):
        self.db = LogDB(":memory:")
        self.db.createTables()

    ################################################
    # expandDomainList
    ################################################
    def test_expandDomainList(self):
        result = expandDomainList(["c", "b.a.a", "a.a.a", "a.a"])
        assert result == ["a.a", "a.a.a", "b.a.a", "c"]

    def test_expandDomainList_skipPublicSuffix(self):
        result = expandDomainList(["doo.foo.bar.co.uk"])
        assert result == [
            "bar.co.uk",
            "co.uk",
            "doo.foo.bar.co.uk",
            "foo.bar.co.uk",
        ]

        result = expandDomainList(
            ["doo.foo.bar.co.uk"], public_suffix_list=["bar.co.uk"]
        )
        assert result == ["doo.foo.bar.co.uk", "foo.bar.co.uk"]

    ################################################
    # logDnsQuery
    ################################################
    def test_logDnsQuery_insertFirst(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = "foo"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == "1.2.3.4, 4.3.2.1"
        assert self.db.DnsChange.get().status_id == status_id
        assert result == status_id

    def test_logDnsQuery_insertOnlyOnePerStatusIdIPUrl(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = "foo"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        answer_list.extend("127.0.0.1")
        try:
            logDnsQuery(
                self.db, status_id, resolver_ip, domain, rdtype, answer_list
            )
        except peewee.IntegrityError:
            assert self.db.DnsChange.select().count() == 1
            assert self.db.DnsChange.get().status_id == status_id
        else:
            raise NotImplementedError("Expected IntegrityError")

    def test_logDnsQuery_skipIdenticalPreviousValues(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = "foo"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        status_id_2 = logStatus(self.db, "foo")
        result_2 = logDnsQuery(
            self.db, status_id_2, resolver_ip, domain, rdtype, answer_list
        )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == "1.2.3.4, 4.3.2.1"
        assert self.db.DnsChange.get().status_id == status_id
        assert result == status_id
        assert result == result_2

    def test_logDnsQuery_insertWhenDifferentAnswer(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = "foo"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        result = logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        answer_list_2 = ["4.3.2.1", "1.2.3.4", "0.0.0.0"]
        status_id_2 = logStatus(self.db, "foo")
        result_2 = logDnsQuery(
            self.db, status_id_2, resolver_ip, domain, rdtype, answer_list_2
        )

        assert result_2 != result
        assert self.db.DnsChange.select().count() == 2

        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id
            ).resolver_ip
            == resolver_ip
        )
        assert (
            self.db.DnsChange.get(self.db.DnsChange.status == status_id).domain
            == domain
        )
        assert (
            self.db.DnsChange.get(self.db.DnsChange.status == status_id).rdtype
            == rdtype
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id
            ).response
            == "1.2.3.4, 4.3.2.1"
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id
            ).status_id
            == status_id
        )

        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id_2
            ).resolver_ip
            == resolver_ip
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id_2
            ).domain
            == domain
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id_2
            ).rdtype
            == rdtype
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id_2
            ).response
            == "0.0.0.0, 1.2.3.4, 4.3.2.1"
        )
        assert (
            self.db.DnsChange.get(
                self.db.DnsChange.status == status_id_2
            ).status_id
            == status_id_2
        )

    def test_logDnsQuery_insertDifferentUrl(self):
        domain = "http://example.org"
        domain_2 = domain + "."
        resolver_ip = "127.0.0.1"
        resolver_ip_2 = resolver_ip + "1"
        rdtype = "foo"
        rdtype_2 = rdtype + "bar"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain, rdtype, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip, domain_2, rdtype, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype_2, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain_2, rdtype, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain, rdtype_2, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip, domain_2, rdtype_2, answer_list
        )
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain_2, rdtype_2, answer_list
        )

        assert self.db.DnsChange.select().count() == 8

    ################################################
    # buildResolver
    ################################################
    def test_buildResolver_default(self):
        resolver = buildResolver("127.0.0.1", 4)
        assert resolver.nameservers == ["127.0.0.1"]
        assert resolver.timeout == 4
        assert resolver.lifetime == 4
        assert resolver.edns == -1

    ################################################
    # queryDNS
    ################################################
    def test_queryDNS_A(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.return_value = [
                MockAnswerA("4.3.2.1"),
                MockAnswerA("1.2.3.4"),
            ]
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == "1.2.3.4, 4.3.2.1"
        assert self.db.DnsChange.get().status_id == status_id
        assert result == ["1.2.3.4", "4.3.2.1"]

    def test_queryDNS_MX(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "MX"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.return_value = [
                MockAnswerMX("mail1.example.org"),
                MockAnswerMX("mail2.example.org"),
            ]
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert (
            self.db.DnsChange.get().response
            == "mail1.example.org, mail2.example.org"
        )
        assert self.db.DnsChange.get().status_id == status_id
        assert result == ["mail1.example.org", "mail2.example.org"]

    def test_queryDNS_TXT(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "TXT"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.return_value = [
                MockAnswerTXT('"foo=bar"'),
                MockAnswerTXT('"v=spf1 -all"'),
            ]
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == '"foo=bar", "v=spf1 -all"'
        assert self.db.DnsChange.get().status_id == status_id
        assert result == ['"foo=bar"', '"v=spf1 -all"']

    def test_queryDNS_rejectRdtype(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "B"
        status_id = logStatus(self.db, "foo")

        try:
            queryDNS(self.db, status_id, resolver_ip, domain, rdtype)
        except AssertionError:
            assert self.db.DnsChange.select().count() == 0
        else:
            raise NotImplementedError("Expected AssertionError")

    def test_queryDNS_nxdomain(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:

            def sideEffect(*args, **kw):
                raise surykatka.dns.dns_resolver.NXDOMAIN()

            mock_query.side_effect = sideEffect
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == ""
        assert self.db.DnsChange.get().status_id == status_id
        assert result == []

    def test_queryDNS_NoAnswer(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:

            def sideEffect(*args, **kw):
                raise surykatka.dns.dns_resolver.NoAnswer()

            mock_query.side_effect = sideEffect
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == ""
        assert self.db.DnsChange.get().status_id == status_id
        assert result == []

    def test_queryDNS_timeout(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:

            def sideEffect(*args, **kw):
                raise surykatka.dns.dns_exception.Timeout()

            mock_query.side_effect = sideEffect
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == ""
        assert self.db.DnsChange.get().status_id == status_id
        assert result == []

    def test_queryDNS_nonameservers(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:

            def sideEffect(*args, **kw):
                raise surykatka.dns.dns_resolver.NoNameservers()

            mock_query.side_effect = sideEffect
            result = queryDNS(self.db, status_id, resolver_ip, domain, rdtype)

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == ""
        assert self.db.DnsChange.get().status_id == status_id
        assert result == []

    ################################################
    # getReachableResolverList
    ################################################
    def test_getReachableResolverList_open(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.return_value = [
                MockAnswerA("4.3.2.1"),
                MockAnswerA("1.2.3.4"),
            ]
            result = getReachableResolverList(
                self.db, status_id, [resolver_ip]
            )

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == "1.2.3.4, 4.3.2.1"
        assert self.db.DnsChange.get().status_id == status_id
        assert result == ["127.0.0.1"]

        assert self.db.NetworkChange.select().count() == 1
        assert self.db.NetworkChange.get().ip == resolver_ip
        assert self.db.NetworkChange.get().port == 53
        assert self.db.NetworkChange.get().transport == "UDP"
        assert self.db.NetworkChange.get().state == "open"
        assert self.db.NetworkChange.get().status_id == status_id

    def test_getReachableResolverList_closed(self):
        resolver_ip = "127.0.0.1"
        domain = "example.org"
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:

            def sideEffect(*args, **kw):
                raise surykatka.dns.dns_exception.Timeout()

            mock_query.side_effect = sideEffect

            result = getReachableResolverList(
                self.db, status_id, [resolver_ip]
            )

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain, rdtype, raise_on_no_answer=False
            )

        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == ""
        assert self.db.DnsChange.get().status_id == status_id
        assert result == []

        assert self.db.NetworkChange.select().count() == 1
        assert self.db.NetworkChange.get().ip == resolver_ip
        assert self.db.NetworkChange.get().port == 53
        assert self.db.NetworkChange.get().transport == "UDP"
        assert self.db.NetworkChange.get().state == "closed"
        assert self.db.NetworkChange.get().status_id == status_id

    ################################################
    # getDomainIpDict
    ################################################
    def test_getDomainIpDict_mutipleDomainIp(self):
        resolver_ip_list = ["127.0.0.1"]
        domain_list = ["example.org"]
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.return_value = [
                MockAnswerA("4.3.2.1"),
                MockAnswerA("1.2.3.4"),
            ]
            result = getDomainIpDict(
                self.db, status_id, resolver_ip_list, domain_list, rdtype
            )

            assert mock_query.call_count == 1
            mock_query.assert_called_with(
                domain_list[0], rdtype, raise_on_no_answer=False
            )

        assert result == {
            "1.2.3.4": ["example.org"],
            "4.3.2.1": ["example.org"],
        }
        assert self.db.DnsChange.select().count() == 1
        assert self.db.NetworkChange.select().count() == 0

    def test_getDomainIpDict_mutipleDNSIp(self):
        resolver_ip_list = ["127.0.0.1", "127.0.0.2"]
        domain_list = ["example.org"]
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.side_effect = [
                [MockAnswerA("4.3.2.1"), MockAnswerA("1.2.3.4")],
                [MockAnswerA("4.3.2.1"), MockAnswerA("1.2.3.5")],
            ]
            result = getDomainIpDict(
                self.db, status_id, resolver_ip_list, domain_list, rdtype
            )

            assert mock_query.call_count == 2
            assert mock_query.call_args_list[0] == mock.call(
                domain_list[0], rdtype, raise_on_no_answer=False
            )
            assert mock_query.call_args_list[1] == mock.call(
                domain_list[0], rdtype, raise_on_no_answer=False
            )

        assert result == {
            "1.2.3.4": ["example.org"],
            "4.3.2.1": ["example.org"],
            "1.2.3.5": ["example.org"],
        }
        assert self.db.DnsChange.select().count() == 2
        assert self.db.NetworkChange.select().count() == 0

    def test_getDomainIpDict_mutipleDomain(self):
        resolver_ip_list = ["127.0.0.1"]
        domain_list = ["example.org", "foo.example.org"]
        rdtype = "A"
        status_id = logStatus(self.db, "foo")

        with mock.patch(
            "surykatka.dns.dns_resolver.Resolver.query"
        ) as mock_query:
            mock_query.side_effect = [
                [MockAnswerA("4.3.2.1"), MockAnswerA("1.2.3.4")],
                [MockAnswerA("4.3.2.1"), MockAnswerA("1.2.3.5")],
            ]
            result = getDomainIpDict(
                self.db, status_id, resolver_ip_list, domain_list, rdtype
            )

            assert mock_query.call_count == 2
            assert mock_query.call_args_list[0] == mock.call(
                domain_list[0], rdtype, raise_on_no_answer=False
            )
            assert mock_query.call_args_list[1] == mock.call(
                domain_list[1], rdtype, raise_on_no_answer=False
            )

        assert result == {
            "1.2.3.4": ["example.org"],
            "4.3.2.1": ["example.org", "foo.example.org"],
            "1.2.3.5": ["foo.example.org"],
        }
        assert self.db.DnsChange.select().count() == 2
        assert self.db.NetworkChange.select().count() == 0

    ################################################
    # packDns
    ################################################
    def test_packDns_oldLog(self):
        domain = "http://example.org"
        resolver_ip = "127.0.0.1"
        rdtype = "foo"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        answer_list_2 = ["4.3.2.1", "1.2.3.4", "0.0.0.0"]
        status_id_2 = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id_2, resolver_ip, domain, rdtype, answer_list_2
        )

        result = packDns(self.db)
        assert self.db.DnsChange.select().count() == 1
        assert self.db.DnsChange.get().resolver_ip == resolver_ip
        assert self.db.DnsChange.get().domain == domain
        assert self.db.DnsChange.get().rdtype == rdtype
        assert self.db.DnsChange.get().response == "0.0.0.0, 1.2.3.4, 4.3.2.1"
        assert self.db.DnsChange.get().status_id == status_id_2
        assert result == None

    def test_packDns_keepDifferentUrl(self):
        domain = "http://example.org"
        domain_2 = domain + "."
        resolver_ip = "127.0.0.1"
        resolver_ip_2 = resolver_ip + "1"
        rdtype = "foo"
        rdtype_2 = rdtype + "bar"
        answer_list = ["4.3.2.1", "1.2.3.4"]
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain, rdtype, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain_2, rdtype, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain, rdtype_2, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain_2, rdtype, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain, rdtype_2, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip, domain_2, rdtype_2, answer_list
        )
        status_id = logStatus(self.db, "foo")
        logDnsQuery(
            self.db, status_id, resolver_ip_2, domain_2, rdtype_2, answer_list
        )

        result = packDns(self.db)
        assert self.db.DnsChange.select().count() == 8
        assert result == None


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SurykatkaDNSTestCase))
    return suite


if __name__ == "__main__":
    unittest.main(defaultTest="suite")
